#include "fbscanner.h"

FBScanner::FBScanner()
{
    operatorList << "+" << "-" << "*" << "/" << "\\" << "&" << "@" << "(" << ")" << "[" << "]" << "{" << "}"
            << "?" << "=" << "<" << ">" << "." << ":" << "#" << "" << "" << "" ;

    separatorList << operatorList << " " << "\n" << "\r";
}








QList<FBScanner::TOKEN> FBScanner::scan(QString code)
{
    QList<FBScanner::TOKEN> retList;
    SCANNER_MODE mode = MODE_NORMAL;
    QString symb;
    TOKEN tok;


    for (int i = 0;i < code.count();i++)
    {
        QCharRef c = code[i];
        switch(mode)
        {
        case MODE_NORMAL:
            if (c == ' ')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

            } else if (c == '\n')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "ENDOFSTATEMENT";
                tok.token = ENDOFLINE;
                retList << tok;

            } else if (c == '\r')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "ENDOFSTATEMENT";
                tok.token = ENDOFSTATEMENT;
                retList << tok;

            } else if (c == '+')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "+";
                tok.token = OPERATOR_PLUS;
                retList << tok;

            } else if (c == '-')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "-";
                tok.token = OPERATOR_MINUS;
                retList << tok;

            }  else if (c == '*')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "*";
                tok.token = FBScanner::OPERATOR_ASTERISK;
                retList << tok;

            } else if (c == '/')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "/";
                tok.token = OPERATOR_SLASH;
                retList << tok;

            } else if (c == '=')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "=";
                tok.token = OPERATOR_EQUAL;
                retList << tok;

            } else if (c == '(')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "(";
                tok.token = OPERATOR_ROUNDBRACKET_OPEN;
                retList << tok;

            } else if (c == ')')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = ")";
                tok.token = OPERATOR_ROUNDBRACKET_CLOSE;
                retList << tok;

            } else if (c == '\\')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "\\";
                tok.token = OPERATOR_BACKSLASH;
                retList << tok;

            } else if (c == '&')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "&";
                tok.token = OPERATOR_AMPERSAND;
                retList << tok;

            } else if (c == '?')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "?";
                tok.token = OPERATOR_QUESTIONMARK;
                retList << tok;

            }  else if (c == '#')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "#";
                tok.token = OPERATOR_HASH;
                retList << tok;

            }  else if (c == '.')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = ".";
                tok.token = OPERATOR_DOT;
                retList << tok;

            }  else if (c == '^')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "^";
                tok.token = OPERATOR_EXP;
                retList << tok;

            } else if (c == '<')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = "<";
                tok.token = OPERATOR_LESSTHAN;
                retList << tok;

            } else if (c == '>')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = ">";
                tok.token = OPERATOR_GREATERTHEN;
                retList << tok;

            } else if (c == ',')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                tok.value = ",";
                tok.token = OPERATOR_KOMMA;
                retList << tok;

            } else if (c == '\"')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                mode = MODE_STRING;
                symb = "\"";

            } else if (c == '\'')
            {
                tok.value = symb;
                tok.token = tokenize(symb);
                if (tok.token != INVALID)
                    retList << tok;

                mode = MODE_COMMENT;
                symb = "\'";

            } else if (c.isDigit() == true && symb == "")
            {
                symb = "";
                mode = MODE_NUMBER;
                symb += c;

            } else
            {
                symb += c;
            }

            continue;


        case MODE_STRING:
            if (c == '\"')
            {
                symb += c;
                tok.value = symb;
                tok.token = STRING;
                retList << tok;

                mode = MODE_NORMAL;
                symb = "";

            } else
            {
                symb += c;
            }

            if (c == '\\' && i+1 < code.count())
            {
                if (code[i+1] == '\"')
                {
                    i++;
                    symb += "\"";
                }

            }

            continue;


        case MODE_COMMENT:
            if (c == '\n')
            {
                tok.value = symb;
                tok.token = COMMENT_SINGLE;
                retList << tok;

                tok.value = "ENDOFSTATEMENT";
                tok.token = ENDOFSTATEMENT;
                retList << tok;

                symb = "";
                mode = MODE_NORMAL;

            } else
            {
                symb += c;
            }

            continue;


        case MODE_NUMBER:
            if (c.isDigit() == true)
            {
                symb += c;
            } else
            {
                tok.value = symb;
                tok.token = NUMBER;
                retList << tok;

                i--;
                mode = MODE_NORMAL;
                symb = "";
            }

            continue;
        }
    }

    return retList;
}







FBScanner::TOKEN_ENUM FBScanner::tokenize(QString &symbol)
{
    QString s = symbol.toLower();
    symbol = "";
    if (s == "declare")
    {
        return KEYWORD_DECLARE;

    } else if (s == "sub")
    {
        return KEYWORD_SUB;

    } else if (s == "function")
    {
        return KEYWORD_FUNCTION;

    } else if (s == "lib")
    {
        return KEYWORD_LIB;

    } else if (s == "alias")
    {
        return KEYWORD_ALIAS;

    } else if (s == "byval")
    {
        return KEYWORD_BYVAL;

    } else if (s == "byref")
    {
        return KEYWORD_BYREF;

    } else if (s == "type")
    {
        return KEYWORD_TYPE;

    } else if (s == "end")
    {
        return KEYWORD_END;

    } else if (s == "slots")
    {
        return KEYWORD_SLOTS;

    } else if (s == "public")
    {
        return KEYWORD_PUBLIC;

    } else if (s == "private")
    {
        return KEYWORD_PRIVATE;

    } else if (s == "as")
    {
        return KEYWORD_AS;

    } else if (s == "ptr")
    {
        return KEYWORD_PTR;

    } else if (s == "pointer")
    {
        return KEYWORD_PTR;

    } else if (s == "")
    {
        return INVALID;

    }


    return IDENTIFIER;
}
